@@ -12,17 +12,18 @@ from functools import partial
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Set
 
-import annofabapi
 import annofabapi.utils
 import dateutil
 import dateutil.parser
 import more_itertools
-from annofabapi.models import InputData, InputDataId, Inspection, JobStatus, JobType, Task, TaskHistory
+from annofabapi.dataclass.task import Task as DcTask
+from annofabapi.models import InputDataId, Inspection, JobStatus, JobType, Task, TaskHistory
 from annofabapi.parser import SimpleAnnotationZipParser
 
 from annofabcli.common.dataclasses import WaitOptions
 from annofabcli.common.download import DownloadingFile
 from annofabcli.common.exceptions import DownloadingFileNotFoundError
+from annofabcli.common.facade import TaskQuery, match_task_with_query
 
 logger = logging.getLogger(__name__)
 
@@ -43,12 +44,24 @@ class Query:
     絞り込み条件
     """
 
-    task_query_param: Optional[Dict[str, Any]] = None
-    ignored_task_id_list: Optional[List[str]] = None
+    task_query: Optional[TaskQuery] = None
+    task_id_set: Optional[Set[str]] = None
+    """集計対象タスクのtask_idのSet"""
+    ignored_task_id_set: Optional[Set[str]] = None
+    """集計対象外タスクのtask_idのSet"""
     start_date: Optional[str] = None
     end_date: Optional[str] = None
 
 
+@dataclass(frozen=True)
+class PseudoInputData:
+    """
+    入力データの必要なプロパティのみ定義したクラス
+    """
+
+    input_duration_seconds: Optional[float] = None
+
+
 class Database:
     """
     Annofabから取得した情報に関するデータベース。
@@ -87,6 +100,8 @@ class Database:
         self.task_histories_json_path = Path(f"{self.checkpoint_dir}/task_histories.json")
         self.annotations_zip_path = Path(f"{self.checkpoint_dir}/simple-annotations.zip")
 
+        self.logging_prefix = f"project_id={project_id}"
+
     def __write_checkpoint(self, obj, pickle_file_name):
         """
         チェックポイントファイルに書き込む。書き込み対象のファイルが存在すれば、書き込む前に同階層にバックアップファイルを作成する。
@@ -127,7 +142,7 @@ class Database:
     def read_annotation_summary(
         self, task_list: List[Task], annotation_summary_func: AnnotationSummaryFunc
     ) -> AnnotationDict:
-        logger.debug(f"reading {str(self.annotations_zip_path)}")
+        logger.debug(f"{self.logging_prefix}: reading {str(self.annotations_zip_path)}")
 
         def read_annotation_summary_per_input_data(task_id: str, input_data_id_: str) -> Dict[str, Any]:
             json_path = f"{task_id}/{input_data_id_}.json"
@@ -142,7 +157,7 @@ class Database:
             for task in task_list:
                 task_count += 1
                 if task_count % 1000 == 0:
-                    logger.debug(f"{task_count} / {len(task_list)} 件目を読み込み中")
+                    logger.debug(f"{self.logging_prefix}: {task_count} / {len(task_list)} 件目を読み込み中")
 
                 task_id = task["task_id"]
                 input_data_id_list = task["input_data_id_list"]
@@ -153,14 +168,14 @@ class Database:
                         input_data_dict[input_data_id] = read_annotation_summary_per_input_data(task_id, input_data_id)
                     annotation_dict[task_id] = input_data_dict
                 except Exception as e:  # pylint: disable=broad-except
-                    logger.warning(f"task_id='{task_id}'のJSONファイル読み込みで失敗しました。")
+                    logger.warning(f"{self.logging_prefix}: task_id='{task_id}'のJSONファイル読み込みで失敗しました。")
                     logger.warning(e)
                     continue
 
             return annotation_dict
 
     def read_inspections_from_json(self, task_id_list: List[str]) -> Dict[str, Dict[InputDataId, List[Inspection]]]:
-        logger.debug(f"reading {self.inspection_json_path}")
+        logger.debug(f"{self.logging_prefix}: reading {self.inspection_json_path}")
 
         with open(str(self.inspection_json_path)) as f:
             all_inspections = json.load(f)
@@ -185,24 +200,20 @@ class Database:
 
         return tasks_dict
 
-    def read_input_data_from_json(self, task_list: List[Task]) -> Dict[str, List[InputData]]:
+    def read_input_data_from_json(self) -> Dict[str, PseudoInputData]:
         path = self.input_data_json_path
-        logger.debug(f"reading {path}")
+        logger.debug(f"{self.logging_prefix}: reading {path}")
         with open(str(path)) as f:
             all_input_data_list = json.load(f)
 
-        all_input_data_dict = {e["input_data_id"]: e for e in all_input_data_list}
-        tasks_dict: Dict[str, List[InputData]] = {}
-
-        for task in task_list:
-            task_id = task["task_id"]
-            input_data_id_list = task["input_data_id_list"]
-            tasks_dict[task_id] = [all_input_data_dict[input_data_id] for input_data_id in input_data_id_list]
-
-        return tasks_dict
+        all_input_data_dict = {
+            e["input_data_id"]: PseudoInputData(input_duration_seconds=e["system_metadata"].get("input_duration"))
+            for e in all_input_data_list
+        }
+        return all_input_data_dict
 
     def read_task_histories_from_json(self, task_id_list: Optional[List[str]] = None) -> Dict[str, List[TaskHistory]]:
-        logger.debug(f"reading {self.task_histories_json_path}")
+        logger.debug(f"{self.logging_prefix}: reading {self.task_histories_json_path}")
         with open(str(self.task_histories_json_path)) as f:
             task_histories_dict = json.load(f)
 
@@ -224,9 +235,9 @@ class Database:
             max_job_access=MAX_JOB_ACCESS,
         )
         if result:
-            logger.info(f"アノテーションの更新が完了しました。")
+            logger.info(f"{self.logging_prefix}: アノテーションの更新が完了しました。")
         else:
-            logger.info(f"アノテーションの更新に失敗しました or {MAX_WAIT_MINUTU} 分待っても、更新が完了しませんでした。")
+            logger.info(f"{self.logging_prefix}: アノテーションの更新に失敗しました or {MAX_WAIT_MINUTU} 分待っても、更新が完了しませんでした。")
             return
 
     def wait_for_completion_updated_task_json(self, project_id):
@@ -240,9 +251,9 @@ class Database:
             max_job_access=MAX_JOB_ACCESS,
         )
         if result:
-            logger.info(f"タスク全件ファイルの更新が完了しました。")
+            logger.info(f"{self.logging_prefix}: タスク全件ファイルの更新が完了しました。")
         else:
-            logger.info(f"タスク全件ファイルの更新に失敗しました or {MAX_WAIT_MINUTU} 分待っても、更新が完了しませんでした。")
+            logger.info(f"{self.logging_prefix}: タスク全件ファイルの更新に失敗しました or {MAX_WAIT_MINUTU} 分待っても、更新が完了しませんでした。")
             return
 
     def update_annotation_zip(self, project_id: str, should_update_annotation_zip: bool = False):
@@ -259,18 +270,23 @@ class Database:
 
         project, _ = self.annofab_service.api.get_project(project_id)
         last_tasks_updated_datetime = project["summary"]["last_tasks_updated_datetime"]
-        logger.debug(f"タスクの最終更新日時={last_tasks_updated_datetime}")
+        logger.debug(f"{self.logging_prefix}: タスクの最終更新日時={last_tasks_updated_datetime}")
+        if last_tasks_updated_datetime is None:
+            logger.warning("{self.logging_prefix}: タスクの最終更新日時がNoneなので、アノテーションzipを更新しません。")
+            return
 
         annotation_specs_history = self.annofab_service.api.get_annotation_specs_histories(project_id)[0]
         annotation_specs_updated_datetime = annotation_specs_history[-1]["updated_datetime"]
-        logger.debug(f"アノテーション仕様の最終更新日時={annotation_specs_updated_datetime}")
+        logger.debug(f"{self.logging_prefix}: アノテーション仕様の最終更新日時={annotation_specs_updated_datetime}")
 
         job_list = self.annofab_service.api.get_project_job(
             project_id, query_params={"type": JobType.GEN_ANNOTATION.value, "limit": 1}
         )[0]["list"]
         if len(job_list) > 0:
             job = job_list[0]
-            logger.debug(f"アノテーションzipの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}")
+            logger.debug(
+                f"{self.logging_prefix}: アノテーションzipの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}"
+            )
 
         if should_update_annotation_zip:
             if len(job_list) == 0:
@@ -280,29 +296,31 @@ class Database:
                 job = job_list[0]
                 job_status = JobStatus(job["job_status"])
                 if job_status == JobStatus.PROGRESS:
-                    logger.info(f"アノテーション更新が完了するまで待ちます。")
+                    logger.info(f"{self.logging_prefix}: アノテーション更新が完了するまで待ちます。")
                     self.wait_for_completion_updated_annotation(project_id)
 
                 elif job_status == JobStatus.SUCCEEDED:
                     if dateutil.parser.parse(job["updated_datetime"]) < dateutil.parser.parse(
                         last_tasks_updated_datetime
                     ):
-                        logger.info(f"タスクの最新更新日時よりアノテーションzipの最終更新日時の方が古いので、アノテーションzipを更新します。")
+                        logger.info(f"{self.logging_prefix}: タスクの最新更新日時よりアノテーションzipの最終更新日時の方が古いので、アノテーションzipを更新します。")
                         self.annofab_service.api.post_annotation_archive_update(project_id)
                         self.wait_for_completion_updated_annotation(project_id)
 
                     elif dateutil.parser.parse(job["updated_datetime"]) < dateutil.parser.parse(
                         annotation_specs_updated_datetime
                     ):
-                        logger.info(f"アノテーション仕様の更新日時よりアノテーションzipの最終更新日時の方が古いので、アノテーションzipを更新します。")
+                        logger.info(
+                            f"{self.logging_prefix}: アノテーション仕様の更新日時よりアノテーションzipの最終更新日時の方が古いので、アノテーションzipを更新します。"
+                        )
                         self.annofab_service.api.post_annotation_archive_update(project_id)
                         self.wait_for_completion_updated_annotation(project_id)
 
                     else:
-                        logger.info(f"アノテーションzipを更新する必要がないので、更新しません。")
+                        logger.info(f"{self.logging_prefix}: アノテーションzipを更新する必要がないので、更新しません。")
 
                 elif job_status == JobStatus.FAILED:
-                    logger.info("アノテーションzipの更新に失敗しているので、アノテーションzipを更新します。")
+                    logger.info("{self.logging_prefix}: ノテーションzipの更新に失敗しているので、アノテーションzipを更新します。")
                     self.annofab_service.api.post_annotation_archive_update(project_id)
                     self.wait_for_completion_updated_annotation(project_id)
 
@@ -326,12 +344,14 @@ class Database:
 
         else:
             job = job_list[0]
-            logger.debug(f"タスク全件ファイルの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}")
+            logger.debug(
+                f"{self.logging_prefix}: タスク全件ファイルの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}"
+            )
 
             if should_update_task_json:
                 job_status = JobStatus(job["job_status"])
                 if job_status == JobStatus.PROGRESS:
-                    logger.info(f"タスク全件ファイルの更新が完了するまで待ちます。")
+                    logger.info(f"{self.logging_prefix}: タスク全件ファイルの更新が完了するまで待ちます。")
                     self.wait_for_completion_updated_task_json(project_id)
 
                 elif job_status == JobStatus.SUCCEEDED:
@@ -339,7 +359,7 @@ class Database:
                     self.wait_for_completion_updated_task_json(project_id)
 
                 elif job_status == JobStatus.FAILED:
-                    logger.info("タスク全件ファイルの更新に失敗しているので、タスク全件ファイルを更新します。")
+                    logger.info("{self.logging_prefix}: タスク全件ファイルの更新に失敗しているので、タスク全件ファイルを更新します。")
                     self.annofab_service.api.post_project_tasks_update(project_id)
                     self.wait_for_completion_updated_task_json(project_id)
 
@@ -352,20 +372,22 @@ class Database:
     def _log_annotation_zip_info(self):
         project, _ = self.annofab_service.api.get_project(self.project_id)
         last_tasks_updated_datetime = project["summary"]["last_tasks_updated_datetime"]
-        logger.debug(f"タスクの最終更新日時={last_tasks_updated_datetime}")
+        logger.debug(f"{self.logging_prefix}: タスクの最終更新日時={last_tasks_updated_datetime}")
 
         annotation_specs_history = self.annofab_service.api.get_annotation_specs_histories(self.project_id)[0]
         annotation_specs_updated_datetime = annotation_specs_history[-1]["updated_datetime"]
-        logger.debug(f"アノテーション仕様の最終更新日時={annotation_specs_updated_datetime}")
+        logger.debug(f"{self.logging_prefix}: アノテーション仕様の最終更新日時={annotation_specs_updated_datetime}")
 
         job_list = self.annofab_service.api.get_project_job(
             self.project_id, query_params={"type": JobType.GEN_ANNOTATION.value, "limit": 1}
         )[0]["list"]
         if len(job_list) == 0:
-            logger.debug(f"アノテーションzipの最終更新日時は不明です。")
+            logger.debug(f"{self.logging_prefix}: アノテーションzipの最終更新日時は不明です。")
         else:
             job = job_list[0]
-            logger.debug(f"アノテーションzipの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}")
+            logger.debug(
+                f"{self.logging_prefix}: アノテーションzipの最終更新日時={job['updated_datetime']}, job_status={job['job_status']}"
+            )
 
     def _download_db_file(self, is_latest: bool):
         """
@@ -391,10 +413,16 @@ class Database:
             self.project_id, dest_path=str(self.tasks_json_path), is_latest=is_latest, wait_options=wait_options
         )
         coroutines[INPUT_DATA_JSON_INDEX] = downloading_obj.download_input_data_json_with_async(
-            self.project_id, dest_path=str(self.input_data_json_path), is_latest=is_latest, wait_options=wait_options,
+            self.project_id,
+            dest_path=str(self.input_data_json_path),
+            is_latest=is_latest,
+            wait_options=wait_options,
         )
         coroutines[ANNOTATION_ZIP_INDEX] = downloading_obj.download_annotation_zip_with_async(
-            self.project_id, dest_path=str(self.annotations_zip_path), is_latest=is_latest, wait_options=wait_options,
+            self.project_id,
+            dest_path=str(self.annotations_zip_path),
+            is_latest=is_latest,
+            wait_options=wait_options,
         )
         coroutines[TASK_HISTORY_JSON_INDEX] = downloading_obj.download_task_history_json_with_async(
             self.project_id, dest_path=str(self.task_histories_json_path)
@@ -427,7 +455,9 @@ class Database:
         dt = dt.replace(tzinfo=dateutil.tz.tzlocal())
         return dt
 
-    def read_tasks_from_json(self,) -> List[Task]:
+    def read_tasks_from_json(
+        self,
+    ) -> List[Task]:
         """
         task.jsonからqueryに合致するタスクを調べる
 
@@ -444,31 +474,32 @@ class Database:
             else None
         )
 
-        def filter_task(arg_task: Dict[str, Any]):
-            """AND条件で絞り込む"""
+        def filter_task(arg_task: Dict[str, Any]) -> bool:
+            """
+            検索条件にマッチするかどうか
+
+            Returns:
+                Trueを返すなら検索条件にマッチする
+            """
 
             flag = True
-            if query.task_query_param is not None:
-                if "phase" in query.task_query_param:
-                    flag = flag and arg_task["phase"] == query.task_query_param["phase"]
+            if query.task_query is not None:
+                flag = flag and match_task_with_query(DcTask.from_dict(arg_task), query.task_query)
 
-                if "status" in query.task_query_param:
-                    flag = flag and arg_task["status"] == query.task_query_param["status"]
-
-                if "task_id" in query.task_query_param:
-                    flag = flag and str(query.task_query_param["task_id"]).lower() in str(arg_task["task_id"]).lower()
+            if query.task_id_set is not None:
+                flag = flag and arg_task["task_id"] in query.task_id_set
 
             # 終了日で絞り込む
             # 開始日の絞り込み条件はタスク履歴を見る
             if dt_end_date is not None:
                 flag = flag and (dateutil.parser.parse(arg_task["updated_datetime"]) < dt_end_date)
 
-            if query.ignored_task_id_list is not None:
-                flag = flag and arg_task["task_id"] not in query.ignored_task_id_list
+            if query.ignored_task_id_set is not None:
+                flag = flag and arg_task["task_id"] not in query.ignored_task_id_set
 
             return flag
 
-        logger.debug(f"reading {self.tasks_json_path}")
+        logger.debug(f"{self.logging_prefix}: reading {self.tasks_json_path}")
         with open(str(self.tasks_json_path)) as f:
             all_tasks = json.load(f)
 
@@ -481,7 +512,7 @@ class Database:
                 filtered_task_list, dict_task_histories=dict_task_histories, start_date=self.query.start_date
             )
 
-        logger.debug(f"集計対象のタスク数 = {len(filtered_task_list)}")
+        logger.debug(f"{self.logging_prefix}: 集計対象のタスク数 = {len(filtered_task_list)}")
         return filtered_task_list
 
     @staticmethod
@@ -507,7 +538,14 @@ class Database:
             if task_histories is None or len(task_histories) == 0:
                 return False
 
-            first_started_datetime = task_histories[0]["started_datetime"]
+            has_task_creation = task_histories[0]["account_id"] is None
+            if has_task_creation and len(task_histories) < 2:
+                return False
+
+            # 初回教師付けの開始日時をfirst_started_datetimeとするため「(タスク作成)」の履歴がある場合はスキップする。
+            first_started_datetime = (task_histories[0] if not has_task_creation else task_histories[1])[
+                "started_datetime"
+            ]
             if first_started_datetime is None:
                 return False
             return dateutil.parser.parse(first_started_datetime) >= dt_start_date
@@ -530,7 +568,7 @@ class Database:
         self._log_annotation_zip_info()
 
         # 統計情報の出力
-        account_statistics = self.annofab_service.api.get_account_statistics(self.project_id)[0]
+        account_statistics = self.annofab_service.wrapper.get_account_statistics(self.project_id)
         self.__write_checkpoint(account_statistics, "account_statistics.pickel")
 
         # 労務管理情報の出力
@@ -596,32 +634,32 @@ class Database:
         Annofabからタスク履歴情報を取得する。
         Args:
             all_tasks: 取得対象のタスク一覧
-            ignored_task_ids: 取得しないタスクID List
 
         Returns:
             タスク履歴情報
 
         """
-        if self.query.ignored_task_id_list is None:
-            ignored_task_ids = set()
-        else:
-            ignored_task_ids = set(self.query.ignored_task_id_list)
-
-        tasks = [e for e in all_tasks if e["task_id"] not in ignored_task_ids]
-        logger.info(f"タスク履歴取得対象のタスク数 = {len(tasks)}")
-
         tasks_dict: Dict[str, List[TaskHistory]] = {}
-        if len(tasks) == 0:
+        if len(all_tasks) == 0:
             return tasks_dict
 
-        task_id_list: List[str] = [e["task_id"] for e in tasks]
+        task_id_list: List[str] = [e["task_id"] for e in all_tasks]
         partial_func = partial(_get_task_histories_dict, self.annofab_service.api, self.project_id)
-        with multiprocessing.Pool() as pool:
-            task_index = 0
-            for obj in pool.map(partial_func, task_id_list):
+        process = multiprocessing.current_process()
+        # project_id単位で並列処理が実行場合があるので、デーモンプロセスかどうかを確認する
+        task_index = 0
+        if process.daemon:
+            for task_id in task_id_list:
                 if task_index % 100 == 0:
-                    logger.debug(f"タスク履歴一覧取得中 {task_index} / {len(tasks)} 件目")
-                tasks_dict.update(obj)
+                    logger.debug(f"{self.logging_prefix}: タスク履歴一覧取得中 {task_index} / {len(all_tasks)} 件目")
+                tasks_dict.update(partial_func(task_id))
                 task_index += 1
+        else:
+            with multiprocessing.Pool() as pool:
+                for obj in pool.map(partial_func, task_id_list):
+                    if task_index % 100 == 0:
+                        logger.debug(f"{self.logging_prefix}: タスク履歴一覧取得中 {task_index} / {len(all_tasks)} 件目")
+                    tasks_dict.update(obj)
+                    task_index += 1
 
         return tasks_dict
